@@ -1344,3 +1344,42 @@ class DiskSpace(object):
             raise DiskSpaceError(_("Wrong df output:\n%s") % data)
         else:
             raise DiskSpaceError(str(p.readerr()))
+
+def calculate_exclude(base_dn, exclude=(), include=()):
+    """
+    Вычисление исключений с использованием фильтров include
+    """
+    # корректируем входные данные, чтобы не учитывался первый /
+    exclude = [x[1:] if x.startswith('/') else x for x in exclude]
+    include = [x[1:] if x.startswith('/') else x for x in include]
+    # перебираем все исключения
+    for exclude_dn in exclude:
+        # ищем фильтры для текущего исключения
+        f_include = [x for x in include
+                     if x == exclude_dn or x.startswith("%s/"%exclude_dn)]
+        # если фильтра нет - то исключение добавляется полностью
+        if not f_include:
+            yield exclude_dn
+        else:
+            # пребираем все файлы в указанно исключении
+            for root, dirs, files in os.walk(path.join(base_dn,exclude_dn)):
+                rel_root = root[len(base_dn)+1:]
+                # если файл указан в фильтре, пропускаем его
+                for fn in (path.join(rel_root,x) for x in files):
+                    if fn not in f_include:
+                        yield fn
+                filtered_dn = []
+                for dn in dirs:
+                    rel_dn = path.join(rel_root,dn)
+                    # если директория не используется как часть фильтра - возвращем её
+                    if all(not x.startswith("%s/"%rel_dn) and x != rel_dn
+                           for x in f_include):
+                        filtered_dn.append(dn)
+                        yield rel_dn
+                    elif rel_dn in f_include:
+                            filtered_dn.append(dn)
+
+                # удаляем из обрабатываемых дирекоторий не использующиеся в фильтре
+                # или указанную конечную директорию
+                for dn in filtered_dn:
+                    dirs.remove(dn)
